-- SPDX-License-Identifier: CC0-1.0

let Genode = env:DHALL_GENODE

in  { config =
        Genode.Init::{
        , verbose = True
        , children =
            toMap
              { ram_fs =
                  Genode.Init.Start::{
                  , binary = "ram_fs"
                  , resources = { caps = 96, ram = Genode.units.MiB 4 }
                  , provides = [ "File_system" ]
                  , config =
                      Some
                        ( Genode.Prelude.XML.text
                            ''
                            <config>
                            	<content>
                            		<dir name="test-fs_report">
                            			<inline name="devices">
                            				<devices version="initial"/>
                            			</inline>
                            		</dir>
                            	</content>
                            	<policy label_prefix="fs_report -> " root="/" writeable="yes"/>
                            	<policy label_prefix="fs_rom -> "    root="/test-fs_report"/>
                            	<policy label_prefix="test-fs_report -> " root="/test-fs_report" writeable="yes"/>
                            </config>
                            ''
                        )
                  }
              , fs_report =
                  Genode.Init.Start::{
                  , binary = "fs_report"
                  , resources = { caps = 96, ram = Genode.units.MiB 4 }
                  , provides = [ "Report" ]
                  , routes =
                      [ Genode.ServiceRoute.child "File_system" "ram_fs" ]
                  , config =
                      Some
                        ( Genode.Prelude.XML.text
                            "<config> <vfs> <fs/> </vfs> </config>"
                        )
                  }
              , fs_rom =
                  Genode.Init.Start::{
                  , binary = "fs_rom"
                  , resources = { caps = 96, ram = Genode.units.MiB 4 }
                  , provides = [ "ROM" ]
                  , routes =
                      [ Genode.ServiceRoute.child "File_system" "ram_fs" ]
                  }
              , test-fs_report =
                  Genode.Init.Start::{
                  , binary = "test-fs_report"
                  , exitPropagate = True
                  , resources = { caps = 96, ram = Genode.units.MiB 4 }
                  , provides = [ "Report" ]
                  , config =
                      Some
                        ( Genode.Prelude.XML.text
                            "<config> <vfs> <fs/> </vfs> </config>"
                        )
                  , routes =
                      [ Genode.ServiceRoute.childLabel
                          "ROM"
                          "fs_rom"
                          (Some "devices")
                          (None Text)
                      , Genode.ServiceRoute.childLabel
                          "ROM"
                          "fs_rom"
                          (Some "focus")
                          (None Text)
                      , Genode.ServiceRoute.child "Report" "fs_report"
                      , Genode.ServiceRoute.parent "Timer"
                      ]
                  }
              }
        }
    , rom =
        let manifest = env:MANIFEST
        
        in  Genode.Boot.toRomPaths
              [ manifest.os.bin.fs_report
              , manifest.os.bin.fs_rom
              , manifest.os.bin.ram_fs
              , manifest.os.bin.test-fs_report
              , manifest.os.lib.vfs
              ]
    }
