-- SPDX-License-Identifier: CC0-1.0

let Genode = env:DHALL_GENODE

let manifest = env:MANIFEST

in  { config =
        Genode.Init::{
        , children =
            toMap
              { nic =
                  Genode.Init.Start::{
                  , binary = "nic_loopback"
                  , provides = [ "Nic" ]
                  }
              , bridge =
                  Genode.Init.Start::{
                  , binary = "nic_bridge"
                  , resources = { caps = 200, ram = Genode.units.MiB 8 }
                  , provides = [ "Nic" ]
                  , routes = [ Genode.ServiceRoute.child "Nic" "nic" ]
                  , config =
                      Some
                        ( Genode.Prelude.XML.text
                            ''
                            <config mac="02:02:02:02:03:00" verbose="no">
                            	<policy label="solo5 -> service0" ip_addr="10.0.0.2"/>
                            	<policy label="solo5 -> service1" ip_addr="10.1.0.2"/>
                            	<default-policy/>
                            </config>
                            ''
                        )
                  }
              , solo5 =
                  Genode.Init.Start::{
                  , binary = "solo5-test_net_2if"
                  , resources = { caps = 256, ram = Genode.units.MiB 5 }
                  , routes =
                      [ Genode.ServiceRoute.parent "Timer"
                      , Genode.ServiceRoute.child "Nic" "bridge"
                      ]
                  , config =
                      Some
                        ( Genode.Prelude.XML.text
                            ''
                            <config><cmdline>limit</cmdline></config>
                            ''
                        )
                  }
              , clients =
                  Genode.Init.Start::{
                  , binary = "sequence"
                  , exitPropagate = True
                  , resources = { caps = 256, ram = Genode.units.MiB 8 }
                  , routes =
                      [ Genode.ServiceRoute.parent "Timer"
                      , Genode.ServiceRoute.child "Nic" "bridge"
                      ]
                  , config =
                      Some
                        ( Genode.Prelude.XML.text
                            ''
                            <config>
                              <start name="ping0">
                                <binary name="ping"/>
                                <config interface="10.0.0.72/24" dst_ip="10.0.0.2" period_sec="1" count="4"/>
                              </start>
                              <start name="ping1">
                                <binary name="ping"/>
                                <config interface="10.1.0.72/24" dst_ip="10.1.0.2" period_sec="1" count="4"/>
                              </start>
                            </config>
                            ''
                        )
                  }
              }
        }
    , rom =
        Genode.Boot.toRomPaths
          [ manifest.solo5-tests.bin.solo5-test_net_2if
          , manifest.os.bin.sequence
          , manifest.os.bin.nic_bridge
          , manifest.os.bin.nic_loopback
          , manifest.os.bin.ping
          ]
    }
